#########################################################################
# Autoconf support for C++
# Copyright (C) 1997 Eleftherios Gkioulekas <lf@amath.washington.edu>
#  
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
########################################################################

# -------------------------------------------------------------------------
# Use this macro to configure your C compiler
# When called the macro does the following things:
# 1. It finds an appropriate C compiler.
#    If you passed the flag --with-cc=foo then it uses that
#    particular compiler
# 2. Check whether the compiler works.
# 3. Checks whether the compiler accepts the -g 
# -------------------------------------------------------------------------

AC_DEFUN(LF_CONFIGURE_CC,[
  dnl Find out what C compiler to use
  AC_ARG_WITH(cc,
              [  --with-cc=CC            Use another C compiler],
              [ CC=$withval ],
              [ AC_CHECK_PROGS(CC, gcc egcc cc /usr/ucb/cc) ])
  
  dnl Check if the compiler works
  AC_MSG_CHECKING([whether $CC works])
  AC_TRY_COMPILER([main(){return(0);}],lf_cc_works,lf_cc_cross)
  AC_MSG_RESULT($lf_cc_works)
  if test $lf_cc_works = no
  then
    AC_MSG_ERROR([Fatal Error: $CC cannot create executables])
  fi

  dnl Check whether the compiler accepts the -g flag
  LF_CHECK_CC_FLAG(-g)

  dnl Add these useful things
#  CFLAGS="${CFLAGS} -I."
#  LDFLAGS="-L."
 
  dnl Export CC and CFLAGS to the Makefile
  AC_SUBST(CC)
  AC_SUBST(CFLAGS)
])

# -----------------------------------------------------------------
# This macro should be called to configure your C++ compiler.
# When called, the macro does the following things:
# 1. It finds an appropriate C++ compiler
#    If you passed the flag --with-cxx=foo, then it uses that
#    particular compiler
# 2. Checks whether the compiler accepts the -g 
# ------------------------------------------------------------------

AC_DEFUN(LF_CONFIGURE_CXX,[

  dnl Find out what C++ compiler to use
  AC_ARG_WITH(cxx,
              [  --with-cxx=CXX          Use another C++ compiler],
              [ CXX=$withval ],
              [ AC_CHECK_PROGS(CXX, g++ eg++ c++ gcc CC cxx cc++, gcc) ])
  
  dnl Check if the compiler works
  AC_MSG_CHECKING([whether $CXX works])
  AC_LANG_SAVE
  AC_LANG_CPLUSPLUS
  AC_TRY_COMPILER([main(){return(0);}],lf_cxx_works,lf_cxx_cross)
  AC_LANG_RESTORE
  AC_MSG_RESULT($lf_cxx_works)
  if test $lf_cxx_works = no
  then
    AC_MSG_ERROR([Fatal Error: $CXX cannot create executables])
  fi

  dnl Check whether the compiler accepts the -g flag
  LF_CHECK_CXX_FLAG(-g)
  
  dnl
  dnl Make sure that the cpp.h compatibility header file exists
  dnl Also make sure that -I and -L flags are present for the current 
  dnl directory
  dnl

#  LF_INSTALL_CPP_H
#  CXXFLAGS="${CXXFLAGS} -I."
#  LDFLAGS="-L."

  dnl Substitute CXX and CXXFLAGS
  AC_SUBST(CXX)
  AC_SUBST(CXXFLAGS)
])

# --------------------------------------------------------------------------
# Check whether the C++ compiler accepts a certain flag
# If it does it adds the flag to CXXFLAGS
# If it does not then it returns an error to lf_ok
# Usage:
#   LF_CHECK_CXX_FLAG(-flag1 -flag2 -flag3 ...)
# -------------------------------------------------------------------------

AC_DEFUN(LF_CHECK_CXX_FLAG,[
  echo 'void f(){}' > conftest.cc
  for i in $1
  do
    AC_MSG_CHECKING([whether $CXX accepts $i])
    if test -z "`${CXX} $i -c conftest.cc 2>&1`"
    then
      CXXFLAGS="${CXXFLAGS} $i"
      AC_MSG_RESULT(yes)
    else
      AC_MSG_RESULT(no)
    fi
  done
  rm -f conftest.cc conftest.o
])

# --------------------------------------------------------------------------
# Check whether the C compiler accepts a certain flag
# If it does it adds the flag to CFLAGS
# If it does not then it returns an error to lf_ok
# Usage:
#  LF_CHECK_CC_FLAG(-flag1 -flag2 -flag3 ...)
# -------------------------------------------------------------------------

AC_DEFUN(LF_CHECK_CC_FLAG,[
  echo 'void f(){}' > conftest.c
  for i in $1
  do
    AC_MSG_CHECKING([whether $CC accepts $i])
    if test -z "`${CC} $i -c conftest.c 2>&1`"
    then
      CFLAGS="${CFLAGS} $i"
      AC_MSG_RESULT(yes)
    else
      AC_MSG_RESULT(no)
    fi
  done
  rm -f conftest.c conftest.o
])

# --------------------------------------------------------------------------
# Check whether the Fortran compiler accepts a certain flag
# If it does it adds the flag to FFLAGS
# If it does not then it returns an error to lf_ok
# Usage:
#  LF_CHECK_F77_FLAG(-flag1 -flag2 -flag3 ...)
# -------------------------------------------------------------------------

AC_DEFUN(LF_CHECK_F77_FLAG,[
  cat << EOF > conftest.f
c....:++++++++++++++++++++++++
      PROGRAM MAIN
      PRINT*,'Hello World!'
      END
EOF
  for i in $1
  do
    AC_MSG_CHECKING([whether $F77 accepts $i])
    if test -z "`${F77} $i -c conftest.f 2>&1`"
    then
      FFLAGS="${FFLAGS} $i"
      AC_MSG_RESULT(yes)  
    else
      AC_MSG_RESULT(no)
    fi
  done
  rm -f conftest.f conftest.o
])

# --------------------------------------------------------------------------
# Provide the configure script with an --with-opt option that
# sets the optimization level. Call this command AFTER you have
# configured ALL your compilers.
# Usage:
#  LF_SET_OPTIMIZATION
# --------------------------------------------------------------------------

AC_DEFUN(LF_SET_OPTIMIZATION,[
  dnl Check for --with-opt
  AC_MSG_CHECKING([whether user wants optimization])
  AC_ARG_WITH(opt,
              [  --with-opt=N            Use optimization level N],
              [ lf_opt_level=$withval
                AC_MSG_RESULT(yes) ],
              [ lf_opt_level=0
                AC_MSG_RESULT(no) ])
 
  dnl Now check whether the compilers in use accept the corresponding
  dnl -O flag
  if test $lf_opt_level -gt 0
  then
    if test -n "${CC}"
    then
      LF_CHECK_CC_FLAG(-O${lf_opt_level})
    fi
    if test -n "${CXX}"
    then
      LF_CHECK_CXX_FLAG(-O${lf_opt_level})
    fi
    if test -n "${F77}"
    then
      LF_CHECK_F77_FLAG(-O${lf_opt_level})
    fi
  fi
])

# ----------------------------------------------------------------------
# Provide the configure script with an --with-warnings option that
# turns on warnings. Call this command AFTER you have configured ALL your
# compilers. 
# ----------------------------------------------------------------------

AC_DEFUN(LF_SET_WARNINGS,[
  dnl Check for --with-warnings
  AC_MSG_CHECKING([whether user wants warnings])
  AC_ARG_WITH(warnings,
              [  --with-warnings         Turn on warnings],
              [ lf_warnings=yes ], [ lf_warnings=no ])
  AC_MSG_RESULT($lf_warnings)
  
  dnl Warnings for the two main compilers
  cc_warning_flags="-Wall"
  cxx_warning_flags="-Wall -Woverloaded-virtual -Wtemplate-debugging"
  if test $lf_warnings = yes
  then
    if test -n "${CC}"
    then
      LF_CHECK_CC_FLAG($cc_warning_flags)
    fi
    if test -n "${CXX}" 
    then
      LF_CHECK_CXX_FLAG($cxx_warning_flags)
    fi
  fi
])

# ----------------------------------------------------------------------
# The following installs the cpp.h compatibility header file
# The macro was made with the help of shar
# ----------------------------------------------------------------------

AC_DEFUN(LF_USE_CPP_H,[

  dnl
  dnl Check for common C++ portability problems
  dnl
  AC_LANG_SAVE
  AC_LANG_CPLUSPLUS

  dnl Check whether we have bool
  AC_MSG_CHECKING(whether C++ has bool)
  AC_TRY_RUN([main() { bool b1=true; bool b2=false; }],
             [ AC_MSG_RESULT(yes) ],
             [ AC_MSG_RESULT(no)
               AC_DEFINE(CXX_HAS_NO_BOOL) ],
             [ AC_MSG_WARN(Don't cross-compile)]
            )

  dnl Test whether C++ has buggy for-loops
  AC_MSG_CHECKING(whether C++ has buggy scoping in for-loops)
  AC_TRY_COMPILE([#include <iostream.h>], [
for (int i=0;i<10;i++) { }
for (int i=0;i<10;i++) { }
], [ AC_MSG_RESULT(no) ],
   [ AC_MSG_RESULT(yes)
     AC_DEFINE(CXX_HAS_BUGGY_FOR_LOOPS) ])

  dnl Test whether the user wants to enable assertions
  AC_MSG_CHECKING(whether user wants assertions)
  AC_ARG_ENABLE(assert,
                [  --disable-assert        don't use cpp.h assert],
                [ AC_MSG_RESULT(no)  ],
                [ AC_DEFINE(USE_ASSERT)
                  AC_MSG_RESULT(yes) ],
               )

  dnl Done with the portability checks
  AC_LANG_RESTORE

  dnl
  dnl Now install the cpp.h file if necessary
  dnl

  AC_MSG_CHECKING([whether cpp.h is installed])
  if test -f 'cpp.h' ; then
    AC_MSG_RESULT(yes)
  else
    AC_MSG_RESULT(no)
    changequote()dnl
    sed 's/^X//' << 'SHAR_EOF' > 'cpp.h' &&
X
X#ifndef __defined_cpp_h
X#define __defined_cpp_h
X
X//
X// Workaround for compilers with buggy for-loop scoping
X// That's quite a few compilers actually including:
X// Dec Alpha cxx, HP-UX CC and SGI CC.
X// The trivial "if" statement provides the correct scoping to the
X// for loop
X//
X
X#ifdef CXX_HAS_BUGGY_FOR_LOOPS
X#undef for
X#define for if(1) for
X#endif
X
X//
X// Fortran-like integer looping macros
X// these critters depend on the scoping work-around above
X//
X
X#define loop(COUNTER,BEGIN,END)  \
Xfor (int COUNTER = BEGIN ; COUNTER <= END ; COUNTER ## ++)
X
X#define inverse_loop(COUNTER,END,BEGIN) \
Xfor (int COUNTER = END; COUNTER >= BEGIN; COUNTER ## --)
X
X#define integer_loop(COUNTER,BEGIN,END,STEP) \
Xfor (int COUNTER = BEGIN; COUNTER <= END; COUNTER += STEP)
X
X//
X// Class protection levels
X// addictive syntactic sugar to make coding nicer
X//
X
X#define pub public:
X#define pro protected:
X#define pri private:
X
X//
X// Every mathematician would like to know pi
X// so this is as good a place as any to throw it in.
X//
X
Xconst double pi = 3.14159265358979324;
X
X//
X// If the C++ compiler we use doesn't have bool, then
X// the following is a near-perfect work-around. 
X// You must make sure your code does not depend on "int" and "bool"
X// being two different types, in overloading for instance.
X//
X
X#ifdef CXX_HAS_NO_BOOL
X#define bool int
X#define true 1
X#define false 0
X#endif
X
X// A better assert.h that doesn't kill the process during debugging.
X#include <iostream.h>
Xinline void __go_to_hell_and_suffer(char *ex,char *file,int line)
X{ 
X // Say what happened
X cerr << "Assertion failed!" << endl;
X cerr << "expression: " << ex << endl;
X cerr << "file: " << file << endl;
X cerr << "line: " << line << endl;
X
X // Cause a segmentation fault
X double *foo = new double[10];
X int i=0;
X while(1) { foo[i++] = 0; }
X}
X
X// Kill any previous definition of assert
X#ifdef assert
X#undef assert
X#endif
X
X// Conditionally define assert depending on whether it is desired
X#ifdef USE_ASSERT
X#define assert(EX) if ((int) (EX)) ; else __go_to_hell_and_suffer(#EX, __FILE__, __LINE__)
X#else
X#define assert(EX)
X#endif
X
X#endif
X
SHAR_EOF
  : || echo 'restore of cpp.h failed'
  fi
  changequote([,])dnl

  dnl
  dnl Add an -I flag so that we can find cpp.h
  dnl
#  CXXFLAGS="${CXXFLAGS} -I`pwd`"
])

#--------------------------------------------------------------------
# On a few very rare systems, all of the libm.a stuff is already in 
# libc.a. Set compiler flags accordingly. Also, Linux requires the "ieee" 
# library for math to work right (and it must appear before "-lm").
# If your program uses the <math.h> library then in your configure.in 
# file add a call like:
#   LF_MATH_LIBRARY
# and in your Makefile.am add 
#   program_LDADD = .... @MATH_LIBS@
# which will expand to the Right Thing
# This macro is based on ideas from the Tcl 8.0 distribution
#--------------------------------------------------------------------

AC_DEFUN(LF_CHECK_MATH_LIBRARY,[
  MATH_LIBS=""
  AC_CHECK_FUNC(sin, , MATH_LIBS="-lm")
  AC_CHECK_LIB(ieee, main, [MATH_LIBS="-lieee $MATH_LIBS"])
  AC_SUBST(MATH_LIBS)])

#---------------------------------------------------------------------
# This macro tests if the compiler realizes ANSI C++ standard 
# lifetime of temporaries. If so, it defines
# HAVE_ANSI_LIFETIME_OF_TEMPORARIES
# ---------------------------------------------------------------------

AC_DEFUN(LF_CXX_LIFETIME_OF_TEMPORARIES,[
  AC_MSG_CHECKING(whether ${CXX} supports ANSI C++ lifetime of temporaries)
  AC_LANG_SAVE
  AC_LANG_CPLUSPLUS
  AC_TRY_RUN(
  [
static int destructor_called = 0;
struct X {
  int dummy;
  X(int)      {}
  X(const X&) {}
  ~X()        { destructor_called = 1; }
};

int f(const X&)   { return destructor_called; }
X g(const X& x)   { return x; }
X h(void)         { return 1; }
int main()        { return f(g(h())); }
  ],
  [ AC_MSG_RESULT(yes)
    AC_DEFINE(HAVE_ANSI_LIFETIME_OF_TEMPORARIES) ],
  [ AC_MSG_RESULT(no) ],[AC_MSG_ERROR(can't crosscompile. Sorry)])
  AC_LANG_RESTORE
])

#------------------------------------------------------------------------
# This macro tests whether the compiler supports named return values.
# This is a GNU extension that can produce more optimized code if
# it is available. If so, define
# HAVE_NAMED_RETURN_VALUES
#------------------------------------------------------------------------

AC_DEFUN(LF_CXX_NAMED_RETURN_VALUES,[
  AC_MSG_CHECKING(whether ${CXX} supports named return values)
  AC_LANG_SAVE
  AC_LANG_CPLUSPLUS
  AC_TRY_COMPILE([#include <iostream.h>],[
struct X { int f(void); };
int X::f(void) return i; { i = 42; }
  ],
  [ AC_MSG_RESULT(yes)
    AC_DEFINE(HAVE_NAMED_RETURN_VALUES) ],
  [ AC_MSG_RESULT(no) ])
  AC_LANG_RESTORE
])

